# Generated by Django 3.1.14 on 2022-01-20 16:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('idp_user', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userrole',
            name='role',
            field=models.CharField(max_length=140),
        ),
    ]
